//
//   File : kvi_opaquelabel.cpp
//   Last major modification : Thu Jan 23 2000 02:15:12 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include "kvi_opaquelabel.h"
#include "kvi_xutils.h"

#include <qfontmetrics.h>

#define KVI_CHAN_LABEL_BORDER 2

//
//   A non themeable label
//

KviOpaqueLabel::KviOpaqueLabel(QWidget *parent,const char *name)
:QLabel(parent,name)
{
}

KviOpaqueLabel::KviOpaqueLabel(const QString &text,QWidget *parent,const char *name)
:QLabel(text,parent,name)
{
}

KviOpaqueLabel::~KviOpaqueLabel()
{
}

// Kill these methods


void KviOpaqueLabel::setMask(const QBitmap &)
{
}

void KviOpaqueLabel::setMask(const QRegion &)
{
}

void KviOpaqueLabel::setAutoMask(bool)
{
}

//
//   A non themeable (from outside) label
//   Used for the channel labels
//

KviChanLabel::KviChanLabel(const char * text,QWidget *parent,const char *name)
:QWidget(parent,name)
{
	m_bCanEdit = false;
	m_szText = text ? text : "";
	m_textColor = colorGroup().text();
	m_backColor = colorGroup().button();
	m_pPixPointer = 0;
	setBackgroundMode(NoBackground);
}

KviChanLabel::KviChanLabel(QWidget *parent,const char *name)
:QWidget(parent,name)
{
	m_bCanEdit = false;
	m_szText = "";
	m_textColor = colorGroup().text();
	m_backColor = colorGroup().button();
	m_pPixPointer = 0;
	setBackgroundMode(NoBackground);
}

KviChanLabel::~KviChanLabel()
{
}

void KviChanLabel::setBackgroundPixmapPointer(QPixmap * pix)
{
	m_pPixPointer = pix;
	update();
}

void KviChanLabel::setTextColor(QColor &clr)
{
	m_textColor = clr;
	update();
}

void KviChanLabel::setBackColor(QColor &clr)
{
	m_backColor = clr;
	update();
}

void KviChanLabel::paintEvent(QPaintEvent *e)
{
	QPixmap * pix = new QPixmap(width(),height());
	Display * dpy = qt_xdisplay();
	GC the_gc = XCreateGC(dpy,pix->handle(),0,0);

	if(m_pPixPointer)
	{
		XSetTile(dpy,the_gc,m_pPixPointer->handle());
		XSetFillStyle(dpy,the_gc,FillTiled);
	} else {
		XSetForeground(dpy,the_gc,m_backColor.pixel());
		XSetBackground(dpy,the_gc,m_backColor.pixel());
		XSetFillStyle(dpy,the_gc,FillSolid);
	}
	XFillRectangle(dpy,pix->handle(),the_gc,0,0,width(),height());

	kvi_xDraw3dRect(dpy,the_gc,pix->handle(),colorGroup(),0,0,width() - 1,height() - 1,true);

	if(m_szText.len())
	{
		XSetForeground(dpy,the_gc,m_textColor.pixel());
		XSetFont(dpy,the_gc,font().handle());
		XDrawString(dpy,pix->handle(),the_gc,KVI_CHAN_LABEL_BORDER,
						(height() >> 1) + (fontMetrics().height() >> 1) - (fontMetrics().descent() + 1),m_szText.ptr(),m_szText.len());
	}

	XCopyArea(dpy,pix->handle(),this->handle(),the_gc,0,0,width(),height(),0,0);

	delete pix;
	XFreeGC(dpy,the_gc);
}

void KviChanLabel::setText(const char * text)
{
	m_szText = text ? text : "";
	update();
}

void KviChanLabel::mousePressEvent(QMouseEvent *e)
{
	if(e->button() & RightButton)emit rightClicked();
	else QWidget::mousePressEvent(e);
}

void KviChanLabel::mouseDoubleClickEvent(QMouseEvent *)
{
	m_bCanEdit = true;
}

void KviChanLabel::mouseReleaseEvent(QMouseEvent *)
{ 
	if(m_bCanEdit)
	{
		emit doubleClicked();
		m_bCanEdit = false;
	}
}

QSize KviChanLabel::sizeHint()
{
	QFontMetrics fm = fontMetrics();
	return QSize(100,fm.height() + (KVI_CHAN_LABEL_BORDER << 1));
}

#include "m_kvi_opaquelabel.moc"
